@@ -36,9 +36,7 @@
 #include "DialogSettings.h"
 #include "HtmlTranslator.h"
 
-FileParameter::FileParameter(QObject * parent) : AbstractParameter(parent, true), _label(0), _button(0), _dialogMode(InputOutputMode)
-{
-}
+FileParameter::FileParameter(QObject * parent) : AbstractParameter(parent, true), _label(0), _button(0), _dialogMode(InputOutputMode) {}
 
 FileParameter::~FileParameter()
 {
@@ -69,6 +67,10 @@ void FileParameter::addTo(QWidget * widget, int row)
   connect(_button, SIGNAL(clicked()), this, SLOT(onButtonPressed()));
 }
 
+void FileParameter::addToKeypointList(KeypointList &) const {}
+
+void FileParameter::extractPositionFromKeypointList(KeypointList &) {}
+
 QString FileParameter::textValue() const
 {
   return QString("\"%1\"").arg(_value);
@@ -121,6 +123,11 @@ bool FileParameter::initFromText(const char * text, int & textLength)
   return true;
 }
 
+bool FileParameter::isQuoted() const
+{
+  return true;
+}
+
 void FileParameter::onButtonPressed()
 {
   QString folder;
