@@ -36,7 +36,7 @@ FilterThread::FilterThread(QObject * parent, const QString & name, const QString
     : QThread(parent), _command(command), _arguments(arguments), _environment(environment), _images(new cimg_library::CImgList<float>), _imageNames(new cimg_library::CImgList<char>), _name(name),
       _messageMode(mode)
 {
-  ENTERING;
+  // ENTERING;
 #ifdef _IS_MACOS_
   setStackSize(8 * 1024 * 1024);
 #endif
@@ -87,6 +87,12 @@ QStringList FilterThread::gmicStatus() const
   if (list.size()) {
     list[0].remove(0, 1);
     list.back().chop(1);
+    QList<QString>::iterator it = list.begin();
+    while (it != list.end()) {
+      QByteArray array = it->toLocal8Bit();
+      gmic::strreplace_fw(array.data());
+      *it++ = array;
+    }
   }
   return list;
 }
@@ -143,15 +149,7 @@ void FilterThread::run()
   _failed = false;
   QString fullCommandLine;
   try {
-    if (_messageMode == GmicQt::Quiet) {
-      fullCommandLine = QString("v -");
-    } else if (_messageMode >= GmicQt::VerboseLayerName && _messageMode <= GmicQt::VerboseLogFile) {
-      fullCommandLine = QString("v -99");
-    } else if (_messageMode == GmicQt::VeryVerboseConsole || _messageMode == GmicQt::VeryVerboseLogFile) {
-      fullCommandLine = QString("v 0");
-    } else if (_messageMode == GmicQt::DebugConsole || _messageMode == GmicQt::DebugLogFile) {
-      fullCommandLine = QString("debug");
-    }
+    fullCommandLine = QString::fromLocal8Bit(GmicQt::commandFromOutputMessageMode(_messageMode));
     fullCommandLine += QString(" %1 %2").arg(_command).arg(_arguments);
     _gmicAbort = false;
     _gmicProgress = -1;
