@@ -157,9 +157,12 @@ MainWindow::MainWindow(QWidget * parent) : QWidget(parent), ui(new Ui::MainWindo
   QSize layersExtent = LayersExtentProxy::getExtent(ui->inOutSelector->inputMode());
   ui->previewWidget->setFullImageSize(layersExtent);
 
+  _lastPreviewKeypointBurstUpdateTime = 0;
+
   TIMING;
   makeConnections();
   TIMING;
+  _keypointBurstEnabled = true;
 }
 
 MainWindow::~MainWindow()
@@ -417,6 +420,7 @@ void MainWindow::onEscapeKeyPressed()
     } else {
       _processor.cancel();
       ui->previewWidget->displayOriginalImage();
+      ui->tbUpdateFilters->setEnabled(true);
     }
   }
 }
@@ -481,9 +485,9 @@ void MainWindow::makeConnections()
 
   connect(ui->pbFullscreen, SIGNAL(toggled(bool)), this, SLOT(onToggleFullScreen(bool)));
 
-  connect(ui->filterParams, SIGNAL(valueChanged()), ui->previewWidget, SLOT(sendUpdateRequest()));
-
+  connect(ui->filterParams, SIGNAL(valueChanged()), this, SLOT(onParametersChanged()));
   connect(ui->previewWidget, SIGNAL(previewUpdateRequested()), this, SLOT(onPreviewUpdateRequested()));
+  connect(ui->previewWidget, SIGNAL(keypointPositionsChanged(unsigned int, unsigned long)), this, SLOT(onPreviewKeypointsEvent(unsigned int, unsigned long)));
 
   connect(ui->tbZoomIn, SIGNAL(clicked(bool)), this, SLOT(onZoomIn()));
   connect(ui->tbZoomOut, SIGNAL(clicked(bool)), this, SLOT(onZoomOut()));
@@ -513,13 +517,18 @@ void MainWindow::makeConnections()
 }
 
 void MainWindow::onPreviewUpdateRequested()
+{
+  onPreviewUpdateRequested(false);
+}
+
+void MainWindow::onPreviewUpdateRequested(bool synchronous)
 {
   if (!ui->cbPreview->isChecked()) {
     ui->previewWidget->invalidateSavedPreview();
     return;
   }
   _processor.init();
-  if (_filtersPresenter->currentFilter().isNoFilter()) {
+  if (_filtersPresenter->currentFilter().isNoPreviewFilter()) {
     ui->previewWidget->displayOriginalImage();
     return;
   }
@@ -527,7 +536,7 @@ void MainWindow::onPreviewUpdateRequested()
 
   const FiltersPresenter::Filter currentFilter = _filtersPresenter->currentFilter();
   GmicProcessor::FilterContext context;
-  context.requestType = GmicProcessor::FilterContext::PreviewProcessing;
+  context.requestType = synchronous ? GmicProcessor::FilterContext::SynchronousPreviewProcessing : GmicProcessor::FilterContext::PreviewProcessing;
   GmicProcessor::FilterContext::VisibleRect & rect = context.visibleRect;
   ui->previewWidget->normalizedVisibleRect(rect.x, rect.y, rect.w, rect.h);
   context.inputOutputState = ui->inOutSelector->state();
@@ -540,6 +549,7 @@ void MainWindow::onPreviewUpdateRequested()
   context.filterName = currentFilter.plainTextName;
   context.filterCommand = currentFilter.previewCommand;
   context.filterArguments = ui->filterParams->valueString();
+  context.filterHash = currentFilter.hash;
   _processor.setContext(context);
   _processor.execute();
 
@@ -547,9 +557,32 @@ void MainWindow::onPreviewUpdateRequested()
   _okButtonShouldApply = true;
 }
 
+void MainWindow::onPreviewKeypointsEvent(unsigned int flags, unsigned long time)
+{
+  if (flags & PreviewWidget::KeypointMouseReleaseEvent) {
+    ui->filterParams->setKeypoints(ui->previewWidget->keypoints(), true);
+    _lastPreviewKeypointBurstUpdateTime = 0;
+    _processor.resetLastSynchronousExecutionDurationMS();
+  } else {
+    ui->filterParams->setKeypoints(ui->previewWidget->keypoints(), false);
+    if ((flags & PreviewWidget::KeypointBurstEvent) && _keypointBurstEnabled) {
+      ulong msSinceLastBurstEvent = time - _lastPreviewKeypointBurstUpdateTime;
+      _keypointBurstEnabled = ((ulong)_processor.lastSynchronousExecutionDurationMS() <= KEYPOINTS_INTERACTIVE_DELAY_MS);
+      if (_keypointBurstEnabled && (msSinceLastBurstEvent >= (ulong)_processor.lastSynchronousExecutionDurationMS())) {
+        onPreviewUpdateRequested(true);
+        _lastPreviewKeypointBurstUpdateTime = time;
+      }
+    }
+  }
+}
+
 void MainWindow::onPreviewImageAvailable()
 {
   ui->filterParams->setValues(_processor.gmicStatus(), false);
+  // Make sure keypoint positions are synchronized with gmic status
+  if (ui->filterParams->hasKeypoints()) {
+    ui->previewWidget->setKeypoints(ui->filterParams->keypoints());
+  }
   ui->previewWidget->setPreviewImage(_processor.previewImage());
   ui->previewWidget->enableRightClick();
   ui->tbUpdateFilters->setEnabled(true);
@@ -561,18 +594,27 @@ void MainWindow::onPreviewImageAvailable()
 void MainWindow::onPreviewError(QString message)
 {
   ui->previewWidget->setPreviewErrorMessage(message);
+  ui->previewWidget->enableRightClick();
   ui->tbUpdateFilters->setEnabled(true);
   if (_pendingActionAfterCurrentProcessing == CloseAction) {
     close();
   }
 }
 
+void MainWindow::onParametersChanged()
+{
+  if (ui->filterParams->hasKeypoints()) {
+    ui->previewWidget->setKeypoints(ui->filterParams->keypoints());
+  }
+  ui->previewWidget->sendUpdateRequest();
+}
+
 void MainWindow::processImage()
 {
   // Abort any already running thread
   _processor.init();
   const FiltersPresenter::Filter currentFilter = _filtersPresenter->currentFilter();
-  if (currentFilter.isNoFilter()) {
+  if (currentFilter.isNoApplyFilter()) {
     return;
   }
 
@@ -590,8 +632,10 @@ void MainWindow::processImage()
   context.outputMessageMode = DialogSettings::outputMessageMode();
   context.filterName = currentFilter.plainTextName;
   context.filterCommand = currentFilter.command;
+  context.filterHash = currentFilter.hash;
   ui->filterParams->updateValueString(false); // Required to get up-to-date values of text parameters
   context.filterArguments = ui->filterParams->valueString();
+  _processor.setGmicStatusQuotedParameters(ui->filterParams->quotedParameters());
   ui->filterParams->clearButtonParameters();
   _processor.setContext(context);
   _processor.execute();
@@ -660,7 +704,7 @@ void MainWindow::onApplyClicked()
 
 void MainWindow::onOkClicked()
 {
-  if (_filtersPresenter->currentFilter().isNoFilter()) {
+  if (_filtersPresenter->currentFilter().isNoApplyFilter()) {
     close();
   }
   if (_okButtonShouldApply) {
@@ -679,7 +723,7 @@ void MainWindow::onCloseClicked()
       _pendingActionAfterCurrentProcessing = CloseAction;
       connect(&_processor, SIGNAL(noMoreUnfinishedJobs()), this, SLOT(close()));
       ui->progressInfoWidget->showBusyIndicator();
-      ui->previewWidget->setPreviewErrorMessage(tr("Waiting for cancelled jobs..."));
+      ui->previewWidget->setOverlayMessage(tr("Waiting for cancelled jobs..."));
       _processor.cancel();
     } else {
       close();
@@ -712,7 +756,7 @@ void MainWindow::onReset()
     ui->filterParams->setValues(_filtersPresenter->currentFilter().defaultParameterValues, true);
     return;
   }
-  if (!_filtersPresenter->currentFilter().isNoFilter()) {
+  if (!_filtersPresenter->currentFilter().isNoPreviewFilter()) {
     ui->filterParams->reset(true);
   }
 }
@@ -776,7 +820,7 @@ void MainWindow::saveSettings()
     settings.setValue(QString("Config/PanelSize%1").arg(i), splitterSizes.at(i));
   }
   splitterSizes = ui->verticalSplitter->sizes();
-  if (!_filtersPresenter->currentFilter().isNoFilter() && !_filtersPresenter->currentFilter().isInvalid()) {
+  if (!_filtersPresenter->currentFilter().hash.isEmpty() && !_filtersPresenter->currentFilter().isInvalid()) {
     settings.setValue(QString("Config/ParamsVerticalSplitterSizeTop"), splitterSizes.at(0));
     settings.setValue(QString("Config/ParamsVerticalSplitterSizeBottom"), splitterSizes.at(1));
   }
@@ -930,6 +974,7 @@ void MainWindow::activateFilter(bool resetZoom)
 {
   saveCurrentParameters();
   const FiltersPresenter::Filter & filter = _filtersPresenter->currentFilter();
+  _keypointBurstEnabled = true;
   if (filter.hash.isEmpty()) {
     setNoFilter();
   } else {
@@ -939,13 +984,15 @@ void MainWindow::activateFilter(bool resetZoom)
     }
     if (!ui->filterParams->build(filter.name, filter.hash, filter.parameters, savedValues)) {
       _filtersPresenter->setInvalidFilter();
+      ui->previewWidget->setKeypoints(KeypointList());
+    } else {
+      ui->previewWidget->setKeypoints(ui->filterParams->keypoints());
     }
     ui->filterName->setText(QString("<b>%1</b>").arg(filter.name));
     ui->inOutSelector->enable();
     ui->inOutSelector->show();
     ui->inOutSelector->setState(ParametersCache::getInputOutputState(filter.hash), false);
     ui->previewWidget->updateFullImageSizeIfDifferent(LayersExtentProxy::getExtent(ui->inOutSelector->inputMode()));
-    Logger::setMode(DialogSettings::outputMessageMode());
     ui->filterName->setVisible(true);
     ui->tbAddFave->setEnabled(true);
     ui->previewWidget->setPreviewFactor(filter.previewFactor, resetZoom);
@@ -987,6 +1034,7 @@ void MainWindow::showEvent(QShowEvent * event)
     Logger::clear();
   }
   QObject::connect(Updater::getInstance(), SIGNAL(updateIsDone(int)), this, SLOT(onStartupFiltersUpdateFinished(int)));
+  Logger::setMode(DialogSettings::outputMessageMode());
   Updater::setOutputMessageMode(DialogSettings::outputMessageMode());
   int ageLimit;
   {
