@@ -32,10 +32,14 @@
 #include <algorithm>
 #include <functional>
 #include "Common.h"
+#include "DialogSettings.h"
 #include "Globals.h"
+#include "GmicStdlib.h"
 #include "ImageConverter.h"
 #include "ImageTools.h"
 #include "LayersExtentProxy.h"
+#include "Logger.h"
+#include "OverrideCursor.h"
 #include "Utils.h"
 #include "gmic.h"
 
@@ -64,6 +68,9 @@ PreviewWidget::PreviewWidget(QWidget * parent) : QWidget(parent), _cachedOrigina
   qApp->installEventFilter(this);
   _rightClickEnabled = false;
   _originalImageSize = QSize(-1, -1);
+  _movedKeypointOrigin = QPoint(-1, -1);
+  _movedKeypointIndex = -1;
+  setMouseTracking(false);
 }
 
 PreviewWidget::~PreviewWidget()
@@ -80,6 +87,8 @@ const cimg_library::CImg<float> & PreviewWidget::image() const
 void PreviewWidget::setPreviewImage(const cimg_library::CImg<float> & image)
 {
   _errorMessage.clear();
+  _errorImage = QImage();
+  _overlayMessage.clear();
   *_image = image;
   *_savedPreview = image;
   _savedPreviewIsValid = true;
@@ -96,9 +105,25 @@ void PreviewWidget::setPreviewImage(const cimg_library::CImg<float> & image)
   update();
 }
 
+void PreviewWidget::setOverlayMessage(const QString & message)
+{
+  _overlayMessage = message;
+  _paintOriginalImage = false;
+  update();
+}
+
+void PreviewWidget::clearOverlayMessage()
+{
+  _overlayMessage.clear();
+  _paintOriginalImage = false;
+  update();
+}
+
 void PreviewWidget::setPreviewErrorMessage(const QString & message)
 {
   _errorMessage = message;
+  _errorImage = QImage();
+  updateErrorImage();
   _paintOriginalImage = false;
   update();
 }
@@ -168,10 +193,138 @@ void PreviewWidget::updateOriginalImagePosition()
   }
 }
 
+void PreviewWidget::updateErrorImage()
+{
+  cimg_library::CImgList<float> images;
+  cimg_library::CImgList<char> imageNames;
+  images.assign();
+  imageNames.assign();
+  gmic_image<float> image;
+  getOriginalImageCrop(image);
+  image.move_to(images);
+  QString fullCommandLine = QString::fromLocal8Bit(GmicQt::commandFromOutputMessageMode(DialogSettings::outputMessageMode()));
+  fullCommandLine += QString(" _host=%1 _tk=qt").arg(GmicQt::HostApplicationShortname);
+  fullCommandLine += QString(" _preview_width=%1").arg(width());
+  fullCommandLine += QString(" _preview_height=%1").arg(height());
+  fullCommandLine += QString(" gui_error_preview \"%2\"").arg(_errorMessage);
+  try {
+    gmic(fullCommandLine.toLocal8Bit().constData(), images, imageNames, GmicStdLib::Array.constData(), true);
+  } catch (...) {
+    images.assign();
+    imageNames.assign();
+  }
+  if (!images.size() || !images.front()) {
+    _errorImage = QImage(size(), QImage::Format_ARGB32);
+    _errorImage.fill(QColor(40, 40, 40));
+    QPainter painter(&_errorImage);
+    painter.setPen(Qt::green);
+    painter.drawText(QRect(0, 0, _errorImage.width(), _errorImage.height()), Qt::AlignCenter | Qt::TextWordWrap, _errorMessage);
+    return;
+  }
+  QImage qimage;
+  ImageConverter::convert(images.front(), qimage);
+  if (qimage.size() != size()) {
+    _errorImage = qimage.scaled(size());
+  } else {
+    _errorImage = qimage;
+  }
+}
+
+void PreviewWidget::paintKeypoints(QPainter & painter)
+{
+  QPen pen;
+  pen.setColor(Qt::black);
+  pen.setWidth(2);
+  painter.setRenderHint(QPainter::Antialiasing, true);
+  painter.setPen(pen);
+
+  QRect visibleRect = rect() & _imagePosition;
+  KeypointList::reverse_iterator it = _keypoints.rbegin();
+  while (it != _keypoints.rend()) {
+    if (!it->isNaN()) {
+      const KeypointList::Keypoint & kp = *it;
+      const int & radius = kp.actualRadiusFromPreviewSize(_imagePosition.size());
+      QPoint visibleCenter = keypointToVisiblePointInWidget(kp);
+      QPoint realCenter = keypointToPointInWidget(kp);
+      QRect r(visibleCenter.x() - radius, visibleCenter.y() - radius, 2 * radius, 2 * radius);
+      if (visibleRect.contains(realCenter, false)) {
+        painter.setBrush(kp.color);
+        pen.setStyle(Qt::SolidLine);
+      } else {
+        painter.setBrush(kp.color.darker(150));
+        pen.setStyle(Qt::DotLine);
+      }
+      pen.setColor(QColor(0, 0, 0, kp.color.alpha()));
+      painter.setPen(pen);
+      painter.drawEllipse(r);
+#ifdef _GMIC_QT_DEBUG_
+      painter.drawPoint(visibleCenter);
+#endif
+    }
+    ++it;
+  }
+}
+
+int PreviewWidget::keypointUnderMouse(const QPoint & p)
+{
+  KeypointList::iterator it = _keypoints.begin();
+  int index = 0;
+  while (it != _keypoints.end()) {
+    if (!it->isNaN()) {
+      const KeypointList::Keypoint & kp = *it;
+      QPoint center = keypointToVisiblePointInWidget(kp);
+      if (roundedDistance(center, p) <= (kp.actualRadiusFromPreviewSize(_imagePosition.size()) + 2)) {
+        return index;
+      }
+    }
+    ++it;
+    ++index;
+  }
+  return -1;
+}
+
+QPoint PreviewWidget::keypointToPointInWidget(const KeypointList::Keypoint & kp) const
+{
+  return QPoint(std::round(_imagePosition.left() + (_imagePosition.width() - 1) * (kp.x / 100.0f)), std::round(_imagePosition.top() + (_imagePosition.height() - 1) * (kp.y / 100.0f)));
+}
+
+QPoint PreviewWidget::keypointToVisiblePointInWidget(const KeypointList::Keypoint & kp) const
+{
+  QPoint p = keypointToPointInWidget(kp);
+  p.rx() = std::max(std::max(_imagePosition.left(), 0), std::min(p.x(), std::min(rect().left() + rect().width(), _imagePosition.left() + _imagePosition.width())));
+  p.ry() = std::max(std::max(_imagePosition.top(), 0), std::min(p.y(), std::min(rect().top() + rect().height(), _imagePosition.top() + _imagePosition.height())));
+  return p;
+}
+
+QPointF PreviewWidget::pointInWidgetToKeypointPosition(const QPoint & p) const
+{
+  QPointF result(100.0 * (p.x() - _imagePosition.left()) / (float)(_imagePosition.width() - 1), 100.0 * (p.y() - _imagePosition.top()) / (float)(_imagePosition.height() - 1));
+  result.rx() = std::min(300.0, std::max(-200.0, result.x()));
+  result.ry() = std::min(300.0, std::max(-200.0, result.y()));
+  return result;
+}
+
 void PreviewWidget::paintPreview(QPainter & painter)
 {
+  if (!_overlayMessage.isEmpty()) {
+    paintOriginalImage(painter);
+    painter.fillRect(_imagePosition, QColor(40, 40, 40, 150));
+    painter.setPen(Qt::green);
+    painter.drawText(_imagePosition, Qt::AlignCenter | Qt::TextWordWrap, _overlayMessage);
+    return;
+  }
+  if (!_errorMessage.isEmpty()) {
+    if (_errorImage.isNull() || (_errorImage.size() != size())) {
+      updateErrorImage();
+    }
+    painter.drawImage(0, 0, _errorImage);
+    paintKeypoints(painter);
+    return;
+  }
+
   if (!_image->width() && !_image->height()) {
     painter.fillRect(rect(), QBrush(_transparency));
+    paintKeypoints(painter);
     return;
   }
   /*  If preview image has a size different from the original image crop, or
@@ -199,11 +352,7 @@ void PreviewWidget::paintPreview(QPainter & painter)
   QImage qimage;
   ImageConverter::convert(_image->get_resize(_imagePosition.width(), _imagePosition.height(), 1, -100, 1), qimage);
   painter.drawImage(_imagePosition, qimage);
-  if (!_errorMessage.isEmpty()) { // TODO : Check this
-    painter.fillRect(_imagePosition, QColor(40, 40, 40, 150));
-    painter.setPen(Qt::green);
-    painter.drawText(_imagePosition, Qt::AlignCenter | Qt::TextWordWrap, _errorMessage);
-  }
+  paintKeypoints(painter);
 }
 
 void PreviewWidget::paintOriginalImage(QPainter & painter)
@@ -221,6 +370,7 @@ void PreviewWidget::paintOriginalImage(QPainter & painter)
     QImage qimage;
     ImageConverter::convert(image, qimage);
     painter.drawImage(_imagePosition, qimage);
+    paintKeypoints(painter);
   }
 }
 
@@ -314,6 +464,11 @@ bool PreviewWidget::eventFilter(QObject *, QEvent * event)
   return false;
 }
 
+void PreviewWidget::leaveEvent(QEvent *)
+{
+  OverrideCursor::setWaiting(false);
+}
+
 void PreviewWidget::wheelEvent(QWheelEvent * event)
 {
   double degrees = event->angleDelta().y() / 8.0;
@@ -330,7 +485,13 @@ void PreviewWidget::wheelEvent(QWheelEvent * event)
 void PreviewWidget::mousePressEvent(QMouseEvent * e)
 {
   if (e->button() == Qt::LeftButton || e->button() == Qt::MiddleButton) {
-    if (_imagePosition.contains(e->pos())) {
+    int index = keypointUnderMouse(e->pos());
+    if (index != -1) {
+      _movedKeypointIndex = index;
+      _keypointTimestamp = e->timestamp();
+      abortUpdateTimer();
+      _mousePosition = QPoint(-1, -1);
+    } else if (_imagePosition.contains(e->pos())) {
       _mousePosition = e->pos();
       abortUpdateTimer();
     } else {
@@ -341,6 +502,10 @@ void PreviewWidget::mousePressEvent(QMouseEvent * e)
   }
 
   if (_rightClickEnabled && (e->button() == Qt::RightButton)) {
+    if (_imagePosition.contains(e->pos())) {
+      _movedKeypointIndex = keypointUnderMouse(e->pos());
+      _movedKeypointOrigin = e->pos();
+    }
     if (_previewEnabled) {
       displayOriginalImage();
     }
@@ -354,19 +519,40 @@ void PreviewWidget::mousePressEvent(QMouseEvent * e)
 void PreviewWidget::mouseReleaseEvent(QMouseEvent * e)
 {
   if (e->button() == Qt::LeftButton || e->button() == Qt::MiddleButton) {
+    //    if (QApplication::overrideCursor() && (QApplication::overrideCursor()->shape() == Qt::CrossCursor)) {
+    //      QApplication::restoreOverrideCursor();
+    //    }
     if (!isAtFullZoom() && _mousePosition != QPoint(-1, -1)) {
       QPoint move = _mousePosition - e->pos();
       onMouseTranslationInImage(move);
       sendUpdateRequest();
       _mousePosition = QPoint(-1, -1);
     }
+    if (_movedKeypointIndex != -1) {
+      QPointF p = pointInWidgetToKeypointPosition(e->pos());
+      KeypointList::Keypoint & kp = _keypoints[_movedKeypointIndex];
+      kp.setPosition(p);
+      _movedKeypointIndex = -1;
+      emit keypointPositionsChanged(KeypointMouseReleaseEvent, e->timestamp());
+    }
     e->accept();
     return;
   }
 
+  if (e->button() == Qt::RightButton) {
+    if (_movedKeypointIndex != -1 && (_movedKeypointOrigin != e->pos())) {
+      emit keypointPositionsChanged(KeypointMouseReleaseEvent, e->timestamp());
+    }
+    _movedKeypointIndex = -1;
+    _movedKeypointOrigin = QPoint(-1, -1);
+  }
+
   if (_rightClickEnabled && _paintOriginalImage && (e->button() == Qt::RightButton)) {
     if (_previewEnabled) {
-      if (_savedPreviewIsValid) {
+      if (!_errorImage.isNull()) {
+        _paintOriginalImage = false;
+        update();
+      } else if (_savedPreviewIsValid) {
         restorePreview();
         _paintOriginalImage = false;
         update();
@@ -381,6 +567,10 @@ void PreviewWidget::mouseReleaseEvent(QMouseEvent * e)
 
 void PreviewWidget::mouseMoveEvent(QMouseEvent * e)
 {
+  if (hasMouseTracking() && (_movedKeypointIndex == -1)) {
+    int index = keypointUnderMouse(e->pos());
+    OverrideCursor::setPointingHand((_mousePosition == QPoint(-1, -1)) && (index != -1));
+  }
   if (e->buttons() & (Qt::LeftButton | Qt::MiddleButton)) {
     if (!isAtFullZoom() && (_mousePosition != QPoint(-1, -1))) {
       QPoint move = _mousePosition - e->pos();
@@ -389,7 +579,31 @@ void PreviewWidget::mouseMoveEvent(QMouseEvent * e)
         _mousePosition = e->pos();
       }
     }
+    if (_movedKeypointIndex != -1) {
+      QPointF p = pointInWidgetToKeypointPosition(e->pos());
+      KeypointList::Keypoint & kp = _keypoints[_movedKeypointIndex];
+      kp.setPosition(p);
+      update();
+      if (kp.burst) {
+        unsigned char flags = 0;
+        if (e->timestamp() - _keypointTimestamp > 15) {
+          flags |= KeypointBurstEvent;
+        }
+        emit keypointPositionsChanged(flags, e->timestamp());
+        _keypointTimestamp = e->timestamp();
+      } else {
+        emit keypointPositionsChanged(0, e->timestamp());
+      }
+    }
     e->accept();
+  } else if (e->buttons() & Qt::RightButton) {
+    if (_movedKeypointIndex != -1) {
+      QPointF p = pointInWidgetToKeypointPosition(e->pos());
+      KeypointList::Keypoint & kp = _keypoints[_movedKeypointIndex];
+      kp.setPosition(p);
+      update();
+      emit keypointPositionsChanged(0, e->timestamp());
+    }
   } else {
     e->ignore();
   }
@@ -561,6 +775,13 @@ void PreviewWidget::saveVisibleCenter()
   _savedVisibleCenter = _visibleRect.center();
 }
 
+int PreviewWidget::roundedDistance(const QPoint & p1, const QPoint & p2)
+{
+  const double dx = p1.x() - p2.x();
+  const double dy = p1.y() - p2.y();
+  return (int)std::round(std::sqrt(dx * dx + dy * dy));
+}
+
 void PreviewWidget::setPreviewFactor(float filterFactor, bool reset)
 {
   _previewFactor = filterFactor;
@@ -666,6 +887,18 @@ void PreviewWidget::setPreviewEnabled(bool on)
   _previewEnabled = on;
 }
 
+const KeypointList & PreviewWidget::keypoints() const
+{
+  return _keypoints;
+}
+
+void PreviewWidget::setKeypoints(const KeypointList & keypoints)
+{
+  _keypoints = keypoints;
+  setMouseTracking(_keypoints.size());
+  update();
+}
+
 void PreviewWidget::invalidateSavedPreview()
 {
   _savedPreviewIsValid = false;
